<?php

namespace TDevAgency\FilesManager\Commands;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Storage;
use TDevAgency\FilesManager\Helpers\CacheHelper;
use TDevAgency\FilesManager\Helpers\SymlinkHelper;
use TDevAgency\FilesManager\Jobs\FilesManagerSymlinkJob;
use TDevAgency\FilesManager\ResourceModels\TdfmFile;

class SymlinksCommand extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'fm:symlinks {--id=} {--queue=}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command description';

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle(): int
    {
        if ($this->hasOption('id') && $this->option('id') !== null) {
            $files = TdfmFile::where('id', $this->option('id'))->get();
        } else {
            $files = TdfmFile::get();

            if (File::isDirectory(public_path('media'))) {
                File::cleanDirectory(public_path('media'));
            }
        }

        if ($files->isEmpty()) {
            return 0;
        }

        if (!File::isDirectory(public_path('media'))) {
            File::makeDirectory(public_path('media'));
            chmod(public_path('media'), 0775);
        }

        $bar = $this->output->createProgressBar(count($files));

        $bar->start();

        /** @var TdfmFile $file */
        foreach ($files as $file) {
            $bar->advance();
            if ($file->original_name === $file->name) {
                continue;
            }
            // continue if file physically doesn't exists on the server
            if (!Storage::disk('tdfm_original')->exists($file->name)) {
                continue;
            }
            if ($this->hasOption('queue') && (bool)$this->option('queue') === true) {
                dispatch(new FilesManagerSymlinkJob($file));
            } else {
                app(SymlinkHelper::class)->symlinkFile($file);
                app(CacheHelper::class)->removeFileData($file->id);
            }
        }
        $bar->finish();

        return 0;
    }
}
